from __future__ import print_function, unicode_literals

import argparse
import sys
from contextlib import contextmanager

from sqlbag import S

from .migra import Migration
from .statements import UnsafeMigrationException


@contextmanager
def arg_context(x):
    if x == "EMPTY":
        yield None

    else:
        with S(x) as s:
            yield s


def parse_args(args):
    parser = argparse.ArgumentParser(description="Generate a database migration.")
    parser.add_argument(
        "--unsafe",
        dest="unsafe",
        action="store_true",
        help="Prevent migra from erroring upon generation of drop statements.",
    )
    parser.add_argument(
        "--schema",
        dest="schema",
        default=None,
        help="Restrict output to statements for a particular schema",
    )
    parser.add_argument(
        "--create-extensions-only",
        dest="create_extensions_only",
        action="store_true",
        default=False,
        help='Only output "create extension..." statements, nothing else.',
    )
    parser.add_argument(
        "--with-privileges",
        dest="with_privileges",
        action="store_true",
        default=False,
        help="Also output privilege differences (ie. grant/revoke statements)",
    )
    parser.add_argument(
        "--force-utf8",
        dest="force_utf8",
        action="store_true",
        default=False,
        help="Force UTF-8 encoding for output",
    )
    parser.add_argument("dburl_from", help="The database you want to migrate.")
    parser.add_argument(
        "dburl_target", help="The database you want to use as the target."
    )
    return parser.parse_args(args)


def run(args, out=None, err=None):
    schema = args.schema
    if not out:
        out = sys.stdout  # pragma: no cover
    if not err:
        err = sys.stderr  # pragma: no cover
    with arg_context(args.dburl_from) as ac0, arg_context(args.dburl_target) as ac1:
        m = Migration(ac0, ac1, schema=schema)
        if args.unsafe:
            m.set_safety(False)
        if args.create_extensions_only:
            m.add_extension_changes(drops=False)
        else:
            m.add_all_changes(privileges=args.with_privileges)
        try:
            if m.statements:
                if args.force_utf8:
                    print(m.sql.encode("utf8"), file=out)
                else:
                    print(m.sql, file=out)
        except UnsafeMigrationException:
            print(
                "-- ERROR: destructive statements generated. Use the --unsafe flag to suppress this error.",
                file=err,
            )
            return 3

        if not m.statements:
            return 0

        else:
            return 2


def do_command():  # pragma: no cover
    args = parse_args(sys.argv[1:])
    status = run(args)
    sys.exit(status)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['migra']

package_data = \
{'': ['*']}

install_requires = \
['schemainspect>=0.1.1585871494', 'six', 'sqlbag']

extras_require = \
{'pg': ['psycopg2-binary']}

entry_points = \
{'console_scripts': ['migra = migra:do_command']}

setup_kwargs = {
    'name': 'migra',
    'version': '1.0.1585875302',
    'description': 'Like `diff` but for PostgreSQL schemas',
    'long_description': '# migra: Like diff but for Postgres schemas\n\n- ## compare schemas\n- ## autogenerate migration scripts\n- ## autosync your development database from your application models\n- ## make your schema changes testable, robust, and (mostly) automatic\n\n`migra` is a schema diff tool for PostgreSQL, written in Python. Use it in your python scripts, or from the command line like this:\n\n    $ migra postgresql:///a postgresql:///b\n    alter table "public"."products" add column newcolumn text;\n\n    alter table "public"."products" add constraint "x" CHECK ((price > (0)::numeric));\n\n`migra` magically figures out all the statements required to get from A to B.\n\nMost features of PostgreSQL are supported.\n\n**Migra supports PostgreSQL >= 10 only.** Known issues exist with earlier versions. Development resources are limited, and feature support rather than backwards compatibility is prioritised.\n\n## THE DOCS\n\nDocumentation is at [djrobstep.com/docs/migra](https://djrobstep.com/docs/migra).\n\n## Folks, schemas are good\n\nSchema migrations are without doubt the most cumbersome and annoying part of working with SQL databases. So much so that some people think that schemas themselves are bad!\n\nBut schemas are actually good. Enforcing data consistency and structure is a good thing. It’s the migration tooling that is bad, because it’s harder to use than it should be. ``migra`` is an attempt to change that, and make migrations easy, safe, and reliable instead of something to dread.\n\n## Contributing\n\nContributing is easy. [Jump into the issues](https://github.com/djrobstep/migra/issues), find a feature or fix you\'d like to work on, and get involved. Or create a new issue and suggest something completely different. If you\'re unsure about any aspect of the process, just ask.\n\n## Credits\n\n- [https://github.com/djrobstep](djrobstep): initial development, maintenance\n- [https://github.com/alvarogzp](alvarogzp): privileges support\n- [https://github.com/seblucas](seblucas): docker improvements\n- [https://github.com/MOZGIII](MOZGIII): docker support\n- [https://github.com/mshahbazi](mshahbazi): misc fixes and enhancements\n',
    'author': 'Robert Lechte',
    'author_email': 'robertlechte@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://migra.djrobstep.com/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
}


setup(**setup_kwargs)



from __future__ import unicode_literals

from .changes import Changes
from .command import do_command
from .migra import Migration
from .statements import Statements, UnsafeMigrationException

__all__ = [
    "Migration",
    "Changes",
    "Statements",
    "UnsafeMigrationException",
    "do_command",
]
